/*
  TinyCircuits Robot Arm over WiFi Tutorial!

  Robotic Arm: enables the use of the robotic arm with the Blynk joystick. If using the TinyScreen+
  users are able to view the current position of each servo as it is moved.

  Start Button - starts recording from the current position.
  Save Button - saves a step in the recording. Please consider the following two rules when recording steps:
      1. Only one servo may be used for a given step. 
      2. A step can only contain movement in one direction.
  Stop Button - stops the recording and begins playing it back on a loop. The loop alternates between the position the 
  arm was in when the recording was started and the position the arm was in when the recording was stopped.
  Shutdown Button - returns the arm to the shutdown position and stops recording playback, if necessary. If you do not 
  complete the shutdown procedure by pressing this button, the servos will jump to the shutdown position when power is reconnected.

  Written by Zachary Lee for TinyCircuits 7/22/19

  https://TinyCircuits.com
*/

#include <Wire.h>
#include "ServoDriver.h"
#include <ArduinoLowPower.h>
#include <SPI.h>
#include <TinyScreen.h>
#include <WiFi101.h>
#include <BlynkSimpleWiFiShield101.h>

// You should get Auth Token in the Blynk App.
// Go to the Project Settings (nut icon).
char auth[] = "YOUR_AUTH";

// Your WiFi credentials.
// Set password to "" for open networks.
char ssid[] = "YOUR_WIFI_NAME";
char pass[] = "YOUR_WIFI_PASSWORD";

ServoDriver servo(NO_R_REMOVED);//this value affects the I2C address, which can be changed by
//removing resistors R1-R3. Then the corresponding R1_REMOVED,
//R2_REMOVED, R1_R2_REMOVED, R1_R4_REMOVED and so on can be set. //Default is NO_R_REMOVED

#if defined (ARDUINO_ARCH_AVR)
#define SerialMonitorInterface Serial
#elif defined(ARDUINO_ARCH_SAMD)
#define SerialMonitorInterface SerialUSB
#endif

// TinyScreen+ variables
TinyScreen display = TinyScreen(TinyScreenPlus);
int background = TS_8b_Black;

const int INCREMENT = 100; // how far to move the robot arm when a signal from the joystick adapter is present in position units
const int MAX_RECORDING_STEPS = 200; // the maximum number of steps that the user can record. only one servo should be moved per step
const int powerPin = 4;
const int DELAY_BETWEEN_MOVEMENTS = 7; // in milliseconds. the lower this number the faster the servos will move

static int claw = 1153; // represents the current position of the claw servo, updated whenever claw servo is moved
static int shoulder = 2100; // represents the current position of the shoulder servo, updated whenever shoulder servo is moved
static int elbow = 550; // represents the current position of the elbow servo
static int twist = 600; // represents the current position of the twist servo
bool startButtonPressed = false;
bool saveButtonPressed = false;
bool stopButtonPressed = false;
bool shutdownButtonPressed = false;
int shoulderStepRun = 1; // keeps track of how many times the shoulder servo was used during the playback process
int elbowStepRun = 1;
int clawStepRun = 1;
int twistStepRun = 1;
bool firstTime = true; // used to record the starting position of a recording
int whichStep = 1; // keeps track of which step is currently being played back by the recording
bool moveToStart = false; // keeps track of when to notify user that the robot arm is returning to starting position in preperation to shut down
int shoulderStepCounter = 1;  // keeps track of how many times the shoulder servo was used during the recording process
int elbowStepCounter = 1;
int clawStepCounter = 1;
int twistStepCounter = 1;
bool first = false;
int movementCounterArray[MAX_RECORDING_STEPS * 4]; // specifies which servo was used for which step

// Global variables to hold joystick data
int RX = 0;
int RY = 0;
int LX = 0;
int LY = 0;
byte leftButton = 0;
byte rightButton = 0;

void setup() {
  display.begin(); // initialize display 
  display.setFlip(true);
  display.setBrightness(15);
  display.setFont(thinPixel7_10ptFontInfo);
  display.fontColor(TS_8b_White, background);
  Wire.begin();
  pinMode(powerPin, OUTPUT);
  digitalWrite(powerPin, HIGH);
  selectPort(2);
  SerialUSB.begin(9600);
  WiFi.setPins(8, 2, A3, -1); // VERY IMPORTANT FOR TINYCIRCUITS WIFI SHIELD
  Blynk.begin(auth, ssid, pass);
  pinMode(9, OUTPUT);//Pin 9 is the reset pin for the servo controller TinyShield
  digitalWrite(9, LOW);
  delay(10);
  digitalWrite(9, HIGH);
  delay(100);

  if (servo.begin(20000)) {      //Set the period to 20000us or 20ms, correct for driving most servos
    SerialMonitorInterface.println("Motor driver not detected!");
    while (1);
  }
  //The failsafe turns off the PWM output if a command is not sent in a certain amount of time.
  //Failsafe is set in milliseconds- comment or set to 0 to disable
  //servo.setFailsafe(1000);
  // initialze servos to starting position, which will cause jump unless shutdown procedure was completed by user
  servo.setServo(1, 2100);
  servo.setServo(2, 550);
  servo.setServo(3, 1153);
  servo.setServo(4, 600);
}

void loop()
{
  static int movementCounter = 1; // keeps track of the total number of movements, regardless of servo
  static int clawArray[MAX_RECORDING_STEPS] = {}; // keeps track of the position of the claw over time
  static int shoulderArray[MAX_RECORDING_STEPS] = {};
  static int elbowArray[MAX_RECORDING_STEPS] = {};
  static int twistArray[MAX_RECORDING_STEPS] = {};
  static bool playbackMode = false; // set to true after user records process. Process is then played back until lower right button is pressed.
  static int stepNumber = 1;
  static unsigned long lowerLeftButton = 0;

  Blynk.run(); // check for any updates from the joystick or buttons

  if (stepNumber == movementCounter && stepNumber != 0) // reset the current step number being executed once all the steps in the recording have been executed
  {
	shoulderStepRun = 1;
	elbowStepRun = 1;
	clawStepRun = 1;
	twistStepRun = 1;
    stepNumber = 1;
  }

  if (startButtonPressed && !playbackMode)
  {
    display.clearScreen();
    display.setCursor(0, 0);
    if(firstTime){
	clawArray[MAX_RECORDING_STEPS-1] = claw;  // stores the original position of recording
	shoulderArray[MAX_RECORDING_STEPS-1] = shoulder;
	elbowArray[MAX_RECORDING_STEPS-1] = elbow;
	twistArray[MAX_RECORDING_STEPS-1] = twist;
    firstTime = false;
    }
    display.println("Recording started");
  }
  else if (saveButtonPressed && !playbackMode)
  {
    lowerLeftButton = millis();
    if (saveButtonPressed)
    {
		centerRecordingArrays(clawArray, shoulderArray, elbowArray, twistArray, movementCounter, playbackMode);  // reserve 0th array position
    }
    if (clawArray[0] == claw && shoulderArray[0] == shoulder && elbowArray[0] == elbow && twistArray[0] == twist) // the user must move at least one of the servos before saving a new step. They can move all of the servos
      // if they wish to do so.
    {
      display.clearScreen();
      display.setCursor(0, 0);
      display.println("Please move the");
      display.setCursor(0, 10);
      display.println("servos before");
      display.setCursor(0, 20);
      display.println("saving another");
      display.setCursor(0, 30);
      display.println("step!");
    }
    else
    {
      if (clawMoved(clawArray, movementCounter))
      {
        clawArray[clawStepCounter] = claw; // only update the value in an array if it was moved
        movementCounterArray[movementCounter] = 3; // servoNumber
        ++clawStepCounter;
        ++movementCounter;
      }
      if (shoulderMoved(shoulderArray, movementCounter))
      {
        shoulderArray[shoulderStepCounter] = shoulder;
        movementCounterArray[movementCounter] = 1; // servoNumber
        ++shoulderStepCounter;
        ++movementCounter;
      }
      if (elbowMoved(elbowArray, movementCounter))
      {
        elbowArray[elbowStepCounter] = elbow;
        movementCounterArray[movementCounter] = 2; // servoNumber
        ++elbowStepCounter;
        ++movementCounter;
      }
      if (twistMoved(twistArray, movementCounter))
      {
        twistArray[twistStepCounter] = twist;
        movementCounterArray[movementCounter] = 4; // servoNumber
        ++twistStepCounter;
        ++movementCounter;
      }

      display.clearScreen();
      display.setCursor(0, 0);
      display.print("Step ");
      display.print(movementCounter-1); // movementCounter has index 1
      display.println(" recorded");
      display.setCursor(0, 10);
      display.println("successfully!");
    }
  }
  else if (stopButtonPressed && !playbackMode && lowerLeftButton != 0)
  {
	  bool finish = true;
	  if (movementCounter == 1)
	  {
		  display.clearScreen();
		  display.setCursor(0, 0);
		  display.println("Please record at");
		  display.setCursor(0, 10);
		  display.println("least one step");
		  display.setCursor(0, 20);
		  display.println("before saving your");
		  display.setCursor(0, 30);
		  display.println("recording!");
		  finish = false;
	  }
	if (stopButtonPressed);    // wait until user stops pressing
	if (finish) {
		display.clearScreen();
		display.setCursor(0, 0);
		display.println("Recording complete!");
		playbackMode = true;
	}
  }
  else if (shutdownButtonPressed)
  {
	 if (shutdownButtonPressed);  // wait until user stops pressing

    display.clearScreen();
    display.setCursor(0, 0);
    display.println("Recording deleted!");
    centerRecordingArrays(clawArray, shoulderArray, elbowArray, twistArray, movementCounter, playbackMode);
    moveToStartPosition(clawArray, shoulderArray, elbowArray, twistArray, movementCounter, playbackMode,stepNumber); // Used to prevent servo jumping to start location later
    resetRecordingArrays(clawArray, shoulderArray, elbowArray, twistArray, movementCounter, playbackMode);
    firstTime = true;
    display.clearScreen();
    display.setCursor(0, 0);
    display.println("Safe to turn off!");
  }

  if (playbackMode) // we need to call these methods one step at a time. remember, a step can only contain the movement of one servo type. This is accounted for with the movementCounterarray[stepNumber] check
  {
      ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
      if (stepNumber < movementCounter && movementCounterArray[stepNumber] == 1) // stepNumber is local loop variable that will be reset after all steps have been iterated through
      {
        moveServosSlowly(shoulderArray, shoulder, movementCounter, playbackMode, 1,stepNumber); // deal with each servo type individually
        if (!moveToStart)
        {
          ++stepNumber;
        }
      }
      else if (stepNumber < movementCounter && movementCounterArray[stepNumber] == 2) // stepNumber is local loop variable that will be reset after all steps have been iterated through
      {
        moveServosSlowly(elbowArray, elbow, movementCounter, playbackMode, 2,stepNumber);
        if (!moveToStart)
        {
          ++stepNumber;
        }
      }
      else if (stepNumber < movementCounter && movementCounterArray[stepNumber] == 3) // stepNumber is local loop variable that will be reset after all steps have been iterated through
      {
        moveServosSlowly(clawArray, claw, movementCounter, playbackMode, 3,stepNumber);
        if (!moveToStart)
        {
          ++stepNumber;
        }
      }
      else if (stepNumber < movementCounter && movementCounterArray[stepNumber] == 4) // stepNumber is local loop variable that will be reset after all steps have been iterated through
      {
        moveServosSlowly(twistArray, twist, movementCounter, playbackMode, 4,stepNumber);
        if (!moveToStart)
        {
          ++stepNumber;
        }
      }
  }

  else // !playbackMode
  {
    if (LY > 200)
    {
      incrementShoulder(DELAY_BETWEEN_MOVEMENTS, INCREMENT, playbackMode);
    }
    else if (LY < -200)
    {
      decrementShoulder(DELAY_BETWEEN_MOVEMENTS, INCREMENT, playbackMode);
    }
    if (RY > 200)
    {
      incrementElbow(DELAY_BETWEEN_MOVEMENTS, INCREMENT, playbackMode);
    }
    else if (RY < -200)
    {
      decrementElbow(DELAY_BETWEEN_MOVEMENTS, INCREMENT, playbackMode);
    }
    if (LX > 200)
    {
      incrementTwist(DELAY_BETWEEN_MOVEMENTS, INCREMENT, playbackMode);
    }
    else if (LX < -200)
    {
      decrementTwist(DELAY_BETWEEN_MOVEMENTS, INCREMENT, playbackMode);
    }

    if (RX > 200)
    {
      incrementClaw(DELAY_BETWEEN_MOVEMENTS, INCREMENT, playbackMode);
    }
    else if (RX < -200)
    {
      decrementClaw(DELAY_BETWEEN_MOVEMENTS, INCREMENT, playbackMode);
    }
  }
}

void incrementClaw(int clawDelay, const int INCREMENT, bool playbackMode)
{
  display.clearScreen();
  int increment = 0;
  while (claw < 1855 && increment < INCREMENT)
  {
    servo.setServo(3, claw);
    ++claw;
    ++increment;
    display.setCursor(0, 0);
    display.print("claw pos: ");
    display.println(claw);
    if (playbackMode || moveToStart)
    {
      displayStatus(0, 10);
    }
    delay(clawDelay);
  }
  return;
}
void decrementClaw(int clawDelay, const int INCREMENT, bool playbackMode)
{
  display.clearScreen();
  int increment = 0;
  while (claw > 450 && increment < INCREMENT)
  {
    servo.setServo(3, claw);
    --claw;
    ++increment;
    display.setCursor(0, 0);
    display.print("claw pos: ");
    display.println(claw);
    if (playbackMode || moveToStart)
    {
      displayStatus(0, 10);
    }
    delay(clawDelay);
  }
  return;
}

void incrementElbow(int extensionDelay, const int INCREMENT, bool playbackMode)
{
  display.clearScreen();
  int increment = 0;
  while (elbow < 2350 && increment < INCREMENT)
  {
    servo.setServo(2, elbow);
    ++elbow;
    ++increment;
    display.setCursor(0, 30);
    display.print("elbow pos: ");
    display.println(elbow);
    if (playbackMode || moveToStart)
    {
      displayStatus(0, 10);
    }
    delay(extensionDelay);
  }
  return;
}

void decrementElbow(int extensionDelay, const int INCREMENT, bool playbackMode)
{
  display.clearScreen();
  int increment = 0;
  while (elbow > 450 && increment < INCREMENT)
  {
    servo.setServo(2, elbow);
    --elbow;
    ++increment;
    display.setCursor(0, 30);
    display.print("elbow pos: ");
    display.println(elbow);
    if (playbackMode || moveToStart)
    {
      displayStatus(0, 10);
    }
    delay(extensionDelay);
  }
  return;
}

void incrementTwist(int twistDelay, const int INCREMENT, bool playbackMode)
{
  display.clearScreen();
  int increment = 0;
  while (twist < 2350 && increment < INCREMENT)
  {
    servo.setServo(4, twist);
    ++twist;
    ++increment;
    display.setCursor(0, 12);
    display.print("twist pos: ");
    display.println(twist);
    if (playbackMode || moveToStart)
    {
      displayStatus(0, 30);
    }
    delay(twistDelay);
  }
  return;

}

void decrementTwist(int twistDelay, const int INCREMENT, bool playbackMode)
{
  int increment = 0;
  while (twist > 450 && increment < INCREMENT)
  {
    servo.setServo(4, twist);
    --twist;
    ++increment;
    display.clearScreen();
    display.setCursor(0, 12);
    display.print("twist pos: ");
    display.println(twist);
    if (playbackMode || moveToStart)
    {
      displayStatus(0, 30);
    }
    delay(twistDelay);
  }
  return;

}

void incrementShoulder(int extensionDelay, const int INCREMENT, bool playbackMode)
{
  display.clearScreen();
  int increment = 0;
  while (shoulder < 2350 && increment < INCREMENT)
  {
    servo.setServo(1, shoulder);
    ++shoulder;
    ++increment;
    display.setCursor(0, 30);
    display.print("should pos: ");
    display.println(shoulder);
    if (playbackMode || moveToStart)
    {
      displayStatus(0, 10);
    }
    delay(extensionDelay);
  }
  return;
}

void decrementShoulder(int extensionDelay, const int INCREMENT, bool playbackMode)
{
  display.clearScreen();
  int increment = 0;
  while (shoulder > 450 && increment < INCREMENT)

  {
    servo.setServo(1, shoulder);
    --shoulder;
    ++increment;
    display.setCursor(0, 30);
    display.print("should pos: ");
    display.println(shoulder);
    if (playbackMode || moveToStart)
    {
      displayStatus(0, 10);
    }
    delay(extensionDelay);
    delay(extensionDelay);
  }
  return;
}

void selectPort(int port) {
  Wire.beginTransmission(0x70);
  Wire.write(0x04 + port);
  Wire.endTransmission();
}

// V0 virtual pin 0 corresponds to left joystick
BLYNK_WRITE(V0){
  LX = -param[0].asInt();
  LY = param[1].asInt();
}

// V1 corresponds to right joystick
BLYNK_WRITE(V1){
  RX = param[0].asInt();
  RY = -param[1].asInt();
}

// V2 corresponds to start button
BLYNK_WRITE(V2){
  if(param.asInt() == 1){
      startButtonPressed = true;
  }
  else{
    startButtonPressed = false;
  }
}

// V3 corresponds to save button
BLYNK_WRITE(V3){
  if(param.asInt() == 1){
      saveButtonPressed = true;
      SerialUSB.println("saveButtonPressed: ");
  }
  else{
    saveButtonPressed = false;
    SerialUSB.println("saveButtonNOTPressed: ");
  }
}
// V4 corresponds to stop button
BLYNK_WRITE(V4){
  if(param.asInt() == 1){
          SerialUSB.println("stopButtonPressed: ");
      stopButtonPressed = true;
  }
  else{
    stopButtonPressed = false;
          SerialUSB.println("stopButtonNOTPressed: ");
  }
}
// V5 corresponds to shutdown button
BLYNK_WRITE(V5){
  if(param.asInt() == 1){
      shutdownButtonPressed = true;
  }
  else{
    shutdownButtonPressed = false;
  }
}
/*// Reading joystick values, storing in global variables
void getJoystick() {
  ///////////////////////////////////////////////////////////////
  Wire.requestFrom(0x22, 6);
  int data[4];
  for (int i = 0; i < 4; i++) {
    data[i] = Wire.read();
  }
  byte lsb = Wire.read();
  byte buttons = ~Wire.read();
  leftButton = buttons & 4;
  rightButton = buttons & 8;
  for (int i = 0; i < 4; i++) {
    data[i] <<= 2;
    data[i] |= ((lsb >> (i * 2)) & 3);
    data[i] -= 511;
  }
  RX = data[0];
  RY = -data[1];
  LX = -data[2];
  LY = data[3];
}*/

void resetRecordingArrays(int clawArray[], int shoulderArray[], int elbowArray[], int twistArray[], int &movementCounter, bool &playbackMode)
{
  for (int i = 0; i < MAX_RECORDING_STEPS; ++i)
  {
      clawArray[i] = 0;
      shoulderArray[i] = 0;
      elbowArray[i] = 0;
      twistArray[i] = 0;
  }
  movementCounter = 1;
  playbackMode = false;
}

void moveServosSlowly(int targetarr[], int current, int &movementCounter, bool playbackMode, int servoPort, int &stepNumber)
{
  if (servoPort == 1 && !moveToStart && playbackMode && !first)
  {
	  whichStep = min(shoulderStepRun, shoulderStepCounter); // find the target position within targetarr by moving through the steps sequentially, but never exceeding array bounds
	  if (targetarr[whichStep] == shoulder)
	  {
		  whichStep = MAX_RECORDING_STEPS-1; // return to orignial position
	  }
    ++shoulderStepRun; 
  }
  else if (servoPort == 2 && !moveToStart && playbackMode && !first)
  {
	  whichStep = min(elbowStepRun, elbowStepCounter);
	  if (targetarr[whichStep] == elbow)
	  {
		  whichStep = MAX_RECORDING_STEPS-1; // return to orignial position
	  }
	  ++elbowStepRun;
  }
  else if (servoPort == 3 && !moveToStart && playbackMode && !first)
  {
	  whichStep = min(clawStepRun, clawStepCounter);
	  if (targetarr[whichStep] == claw)
	  {
		  whichStep = MAX_RECORDING_STEPS-1; // return to orignial position
	  }
    ++clawStepRun;
  }
  else if (servoPort == 4 && !moveToStart && playbackMode && !first)
  {
	  whichStep = min(twistStepRun, twistStepCounter);
	  if (targetarr[whichStep] == twist)
	  {
		  whichStep = MAX_RECORDING_STEPS-1; // return to orignial position
	  }
    ++twistStepRun;
  }
  else if (moveToStart)
  {
	  whichStep = 0; // move to shutdown position
  }

  while (current < targetarr[whichStep] && targetarr[whichStep] - current > 99) // reduce distance between current and target slowly in increments
  {
	if (shutdownButtonPressed)
	{
		Blynk.run(); // ensures that arm still returns to shutdown position if not currently in playback mode
		if (shutdownButtonPressed) {
			return; // if user presses shutdown button, stop moving servos
		}
	}
    if (moveToStart || whichStep == MAX_RECORDING_STEPS-1)
    {
      display.setCursor(0, 40);
      display.println("returning to");
      display.setCursor(0, 50);
      display.println("start position...");
      first = false;
    }
    incrementLogic(playbackMode, servoPort, current); // decides which of the 8 servo methods to call based on servoPort

  }
  if ((targetarr[whichStep] - current) <= 99 && targetarr[whichStep] - current > 0) // once the distance becomes small, use setServo to jump the rest of the way
  {
    servo.setServo(servoPort, targetarr[whichStep]);
  }
  while (current > targetarr[whichStep] && current - targetarr[whichStep] > 99)
  {
	if (shutdownButtonPressed)
	{
		Blynk.run();
		if (shutdownButtonPressed) {
			return; // if user presses shutdown button, stop moving servos
		}
	}
    if (moveToStart || whichStep == MAX_RECORDING_STEPS - 1) {
      display.setCursor(0, 40);
      display.println("returning to");
      display.setCursor(0, 50);
      display.println("start position...");
      first = false;
    }
    decrementLogic(playbackMode, servoPort, current);
  }
  if ((current - targetarr[whichStep]) <= 99 && current - targetarr[whichStep] > 0) // once the distance becomes small, use setServo to jump the rest of the way
  {
    servo.setServo(servoPort, targetarr[whichStep]);
  }
}

void incrementLogic(bool playbackMode, int servoPort, int &current)
{
  if (servoPort == 1)
  {
    incrementShoulder(DELAY_BETWEEN_MOVEMENTS, INCREMENT,  playbackMode);
    current = shoulder;
  }
  else if (servoPort == 2)
  {
    incrementElbow(DELAY_BETWEEN_MOVEMENTS, INCREMENT, playbackMode);
    current = elbow;
  }
  else if (servoPort == 3)
  {
    incrementClaw(DELAY_BETWEEN_MOVEMENTS, INCREMENT, playbackMode);
    current = claw;
  }
  else // servoPort == 4
  {
    incrementTwist(DELAY_BETWEEN_MOVEMENTS, INCREMENT, playbackMode);
    current = twist;
  }
}

void decrementLogic(bool playbackMode, int servoPort, int &current)
{
  if (servoPort == 1)
  {
    decrementShoulder(DELAY_BETWEEN_MOVEMENTS, INCREMENT, playbackMode);
    current = shoulder;
  }
  else if (servoPort == 2)
  {
    decrementElbow(DELAY_BETWEEN_MOVEMENTS, INCREMENT, playbackMode);
    current = elbow;
  }
  else if (servoPort == 3)
  {
    decrementClaw(DELAY_BETWEEN_MOVEMENTS, INCREMENT,  playbackMode);
    current = claw;
  }
  else // servoPort == 4
  {
    decrementTwist(DELAY_BETWEEN_MOVEMENTS, INCREMENT,  playbackMode);
    current = twist;
  }
}

void moveToStartPosition(int clawArray[], int shoulderArray[], int elbowArray[], int twistArray[], int &movementCounter, bool &playbackMode, int &stepNumber)
{
  moveToStart = true;
  moveServosSlowly(shoulderArray, shoulder, movementCounter, playbackMode, 1, stepNumber);
  moveServosSlowly(elbowArray, elbow, movementCounter, playbackMode, 2,stepNumber);
  moveServosSlowly(clawArray, claw, movementCounter, playbackMode, 3,stepNumber);
  moveServosSlowly(twistArray, twist, movementCounter, playbackMode, 4,stepNumber);
  moveToStart = false;
}

void centerRecordingArrays(int clawArray[], int shoulderArray[], int elbowArray[], int twistArray[], int &movementCounter, bool &playbackMode)
{	// startup/shutdown is reserved the 0th array position
	clawArray[0] = 1153;
	shoulderArray[0] = 2100;
	elbowArray[0] = 550;
	twistArray[0] = 600;
}

void displayStatus(int xCoord, int yCoord)
{
  if (!moveToStart)
  {
    display.setCursor(xCoord, yCoord);
    display.println("Playing recording...");
  }
  else if (moveToStart) {
    display.setCursor(xCoord, yCoord);
    display.println("Shutting Down...");
  }
}

bool shoulderMoved(int shoulderArray[], int movementCounter)
{
  if (shoulder != shoulderArray[shoulderStepCounter - 1]) // if the current shoulder value is not equal to the last stored shoulder value.
  {
    return true;
  }
  return false;
}

bool elbowMoved(int elbowArray[], int movementCounter)
{

  if (elbow != elbowArray[elbowStepCounter - 1]) // if the current shoulder value is not equal to the last stored shoulder value.
  {
    return true;
  }
  return false;
}

bool clawMoved(int clawArray[], int movementCounter)
{
  if (claw != clawArray[clawStepCounter - 1]) // if the current shoulder value is not equal to the last stored shoulder value.
  {
    return true;
  }
  return false;
}

bool twistMoved(int twistArray[], int movementCounter)
{
  if (twist != twistArray[twistStepCounter - 1]) // if the current shoulder value is not equal to the last stored shoulder value.
  {
    return true;
  }
  return false;
}
